`define HEADER TRUE
//`define PreProduction_Version TRUE
//`define Simulation TRUE
//`define Slave TRUE


// Reset Enable signals
`define      Internal 5'b00001
`define   Front_Panel 5'b00010
`define  System_Reset 5'b00100
`define    Hard_Reset 5'b01000
`define External_JTAG 5'b10000


// State definitions for ethernet packets
`define    Wait_for_packet 3'o0
`define           Preamble 3'o1
`define           MAC_Dest 3'o2
`define         MAC_Source 3'o3
`define          Store_LEN 3'o4
`define            Payload 3'o5
`define       Update_Stats 3'o6
`define      Wait_for_Idle 3'o7

// Define Octet constants
`define     D5_6 8'hC5        
`define    D16_2 8'h50        
`define    K28_5 8'hBC      
`define    K23_7 8'hF7     // /R/ Carrier Extend
`define    K27_7 8'hFB     // /S/ SOP
`define    K29_7 8'hFD     // /T/ EOP
`define    K30_7 8'hFE     // /V/ ERROR_Prop
//
`define    PRMBL 8'h55     // Preamble octet
`define    SOF_BYTE 8'hD5  // Start of Frame octet
//
`define    IDLE1 {`K28_5,`D5_6}
`define    IDLE2 {`K28_5,`D16_2}
`define    PREAMBLE1 {`K27_7,`PRMBL}     // Start of Packet plus first preamble word 
`define    PREAMBLE2 {`PRMBL,`PRMBL}     // Preamble words 
`define    PREAMBLE3 {`PRMBL,`PRMBL}     // Preamble words 
`define    PREAMBLE4 {`PRMBL,`SOF_BYTE}  // Preamble plus Start of Frame word 
`define    Carrier_Extend {`K23_7,`K23_7}  // Carrier extend 
`define    End_of_Packet {`K29_7,`K23_7}  // End of Packet plus carrier extend 

// Packet Parameters
`define    MAC_BCAST 16'hFFFF
`define    PRMB_TIMEOUT 16'd10
`define    PACKET_TIMEOUT 16'd5000
`define    MAX_BYTE 16'd9000   //Maximum number of bytes in jumbo packets
`define       ZERO16 16'h0000  //16 bit zero
`define       One_13 13'd1     //13 bit 1
`define       Pkt_Space 4'd6   //Minimum inter-packet spacing is 12 bytes (6 words) 
//`define       Pkt_Space 4'hF   //Temporary value using DV1024CLK to give 250uS between packets. 
`define  Min_Frame_Size 6'd32  //Minimum packet size is 64 Bytes (32 words) 
`define        Min_Pkt 16'd46  //16 bit Minimum packet size in Bytes (data payload)
`define    Min_Pkt_Size 6'd23  //Minimum packet size (data payload) is 46 Bytes (23 words) 
`define   Min_Time_Slot 8'd255 //Minimum time slot is 4096 bit times 512 bytes (256 words) 
`define     Header_Size 3'd4   //Header size in words if present 
`define        Attempts 2'd3   //Maximum number of transmit attempts

// State definitions for RX FIFO Write Control
`define          RST_all  3'o0
`define        Save_Addr  3'o1
`define    Wait_for_Data  3'o2
`define Store_MAC_Source  3'o3
`define Store_Frame_Data  3'o4
`define     Bad_Frame_St  3'o5
`define    Good_Frame_St  3'o6
`define         Not_def2  3'o7

// State definitions for RX FIFO Read Control
`define   Wait_for_Frame  2'o0
`define     Load_Cnt_Dwn  2'o1
`define    Issue_Frm_Rdy  2'o2
`define        Not_def_2  3'o3

// Comparison constants
//`define Term_Cnt  13'd6143            // Terminal count for FIFO size
//`define Full_Cnt  13'd6143            // FIFO word count when full
//`define   AF_Cnt  13'd6127            // FIFO word count when almost full
`define Full_Cnt  13'd8191            // FIFO word count when full (Now using 8k FIFOs)
`define   AF_Cnt  13'd8063            // FIFO word count when almost full
`define   Zero_1  13'd0               // 13 bit zero
`define   Zero_2  10'd0               // 10 bit zero
`define RST_hold_off  16'hFFFF        // hold off time to enable RESETs after power up (~1 Sec).
`define Clr_Pndg_Time_Out 24'h2FAF08  // Time out for clearing pending transactions 50 mSec.
// State definitions for TX FIFO Read Control
`define Wait_for_Payload  2'o0
`define    Load_Data_Len  2'o1
`define       Issue_Send  2'o2
// State definitions for TX FIFO Write Control
`define     Standby_1  3'o0
`define      Push_New  3'o1
`define Push_Frag_New  3'o2
`define     Standby_2  3'o3
`define   Push_Frag_1  3'o4
`define   Push_Frag_2  3'o5
// Define count reference points
`define        Hdr_Size 2'd3   //Header size in words if present 
`define Max_Payload 13'd4493-`Hdr_Size // Maximum data payload size in 16 bit words
//`define Max_Payload 13'd28 // Maximum data payload size in 16 bit words

// State definitions for FIFO read state machine
`define          PL_Idle 4'h0
`define   Store_Ctrl_Wrd 4'h1
`define Wait_4_Frame_Req 4'h2
`define        Read_FIFO 4'h3
`define      End_of_Data 4'h4
`define        Rpt_Error 4'h5
`define             Pop1 4'h6
`define             Pop2 4'h7
`define             Pop3 4'h8
`define            Push1 4'h9
`define            Push2 4'hA
`define            Push3 4'hB


// State definitions for Transmit Control
`define Wait_4_TX_Ready 4'h0
`define   Rst_Addr_Cntr 4'h1
`define           Norm1 4'h2
`define           Norm2 4'h3
`define           Norm3 4'h4
`define           Norm4 4'h5
`define           Norm5 4'h6
`define Transmit_Packet 4'h7
`define    Wait_TX_Done 4'h8
`define    Upd_TX_Stats 4'h9

// State definitions for ethernet packets
`define        Xmit_Idle 4'h0
`define      TX_Preamble 4'h1
`define      TX_MAC_Dest 4'h2
`define    TX_MAC_Source 4'h3
`define           Length 4'h4
`define    Proto_Encoder 4'h5
`define     Payload_Data 4'h6
`define     Min_Pkt_Fill 4'h7
`define        CRC_Space 4'h8
`define              EOP 4'h9
`define       Byte_Extnd 4'ha
`define    Int_Pkt_Space 4'hb

// State definitions for ROM address
`define     Wait_for_Sig 4'h0
`define        Preamble1 4'h1
`define        Preamble2 4'h2
`define        Preamble3 4'h3
`define        Preamble4 4'h4
`define            M_P_F 4'h5
`define            CRC_1 4'h6
`define            CRC_2 4'h7
`define            E_O_P 4'h8
`define            B_Ext 4'h9
`define            I_P_S 4'ha

// CRC Checking State Machine
`define       Wait_for_SOP  3'o0
`define    Wait_for_Update  3'o1
`define Wait_for_CRC_Check  3'o2
`define           Good_CRC  3'o3
`define            Bad_CRC  3'o4
`define       No_CRC_Check  3'o5
`define       Record_Stats  3'o6

// Timeout waiting for CRC check
`define    Time_Limit  3'o7


// Mux selection constants
`define   Sel_Constants 3'o0
`define Sel_Destination 3'o1
`define      Sel_Source 3'o2
`define      Sel_Length 3'o3
`define    Sel_Protocol 3'o4
`define        Sel_Data 3'o5

// State definitions for MAC recall
`define Wait_for_SOF 3'h0
`define          MP1 3'h1
`define          MP2 3'h2
`define          MP3 3'h3
`define          MS1 3'h4
`define          MS2 3'h5
`define          MS3 3'h6

// State definitions for Header recall
`define  Wait_for_Len 3'h0
`define          HDR1 3'h1
`define          HDR2 3'h2
`define          HDR3 3'h3
`define          HDR4 3'h4


// Temporary MAC address storage in ROM
//`define    MY_MAC1 8'hAA
//`define    MY_MAC2 8'hBB
//`define    MY_MAC3 8'hCC
//`define    MY_MAC4 8'hDD
//`define    MY_MAC5 8'hEE
//`define    MY_MAC6 8'hFF
//`define    MY_MCAST_MAC1 8'h03
//`define    MY_MCAST_MAC2 8'h00
//`define    MY_MCAST_MAC3 8'h00
//`define    MY_MCAST_MAC4 8'h00
//`define    MY_MCAST_MAC5 8'h00
//`define    MY_MCAST_MAC6 8'h01



// Power on reset FSM State definitions (Runs on the RAW clock):

`define     GSR_Init 3'd0
`define   Reset_DCMs 3'd1
`define   W4DCM_Lock 3'd2
`define FIFO_Startup 3'd3
`define   Pow_on_Rst 3'd4
`define    Run_State 3'd5
`define  Rpt_DCM_Err 3'd6

// Power on reset timeouts
`define  Rst_DCM_TMO 4'hD
`define  POR_TMO     4'h4

// Power up configuration sequence FSM state definitions
`define             PUC 4'd0
`define   Get_Dflt_Cnfg 4'd1
`define        Hrdw_Ini 4'd2
`define         HndShk1 4'd3
`define       Rstr_MACs 4'd4
`define         HndShk2 4'd5
`define       Rstr_Cnfg 4'd6
`define         HndShk3 4'd7
`define    PU_Cnfg_Done 4'd8


// State definitions for Ethernet Startup
`define  Wait_for_Fiber 4'd0
`define    Start_RIO_TX 4'd1
`define        Start_TX 4'd2
`define    Start_RIO_RX 4'd3
`define     Wait_Time_1 4'd4
`define     Holdoff_BTC 4'd5
`define   Snd_Start_Pkt 4'd6
`define         Running 4'd7
`define   Wait_for_Sync 4'd8
`define     Wait_Time_2 4'd9

// Wait time after Sync and Bus traffic controller holdoff
`define Sync_Wait_Time 5'h1A
`ifdef Simulation
   `define    BTC_Holdoff 16'h000F //240nS holdoff for simulations 
`else
   `define    BTC_Holdoff 16'hFFFF //1.04 msec wait for link to be established
`endif

// State definitions for writing MAC addresses
//`define Wait_for_SysStart 2'd0
//`define            My_MAC 2'd1
//`define          My_MCAST 2'd2
//`define          Hold_Off 2'd3

// State definitions for Reset control handling
`define   Wait_for_Shtdwn 2'b00
`define       House_Clean 2'b01
`define  Rdy_for_Shutdown 2'b10

// State definitions for Buss Traffic Controller
`define   Wait_for_Buss_Req  5'h00
`define         Get_Op_Code  5'h01
`define       OR_Req_Busses  5'h02
`define       EX_Req_Busses  5'h03
`define Wait_for_Valid_Data  5'h04
`define        Lp_Back_Data  5'h05
`define        Wait_for_EOD  5'h06
`define       Wait_for_Done  5'h07
`define              Rmv_BR  5'h08
`define       Clr_Funct_Reg  5'h09
`define    Get_RX_Stat_Info  5'h0A
`define          TX_N_Words  5'h0B
`define       R_X_FF_Off_EC  5'h0C
`define          Cnfg2Flash  5'h0D
`define           Read_X_FF  5'h0E
`define             Rd_Done  5'h0F
`define             Send_PP  5'h10
`define           Load_Errs  5'h11
`define          Load_Warns  5'h12
`define          Load_Infos  5'h13
`define     Spnt_Req_Busses  5'h14
`define         TX_Spnt_Pkt  5'h15
`define             Ins_Gap  5'h16
`define            Send_Pkt  5'h17
`define         TC_Clr_Pndg  5'h18
`define          TC_Rdy4Shd  5'h19


// Function definitions for Opcode
`define        Funct_NoOp 8'h00
`define       Set_FF_Test 8'h01
`define        Set_FF_VME 8'h02
`define        ECC_enable 8'h03
`define       ECC_disable 8'h04
`define     Save_Cnfg_Num 8'h05
`define Read_Cnfg_Num_Dir 8'h06
`define Read_Cnfg_Num_Dcd 8'h07
`define     Rstr_Cnfg_Num 8'h08
`define     Set_Cnfg_Dflt 8'h09
`define    Read_Cnfg_Dflt 8'h0A
`define          Set_MACs 8'h0B
`define     Read_MACs_Dir 8'h0C
`define     Read_MACs_Dcd 8'h0D
`define          Read_CRs 8'h0E
`define        Wrt_Eth_CR 8'h0F
`define        Wrt_Ext_CR 8'h10
`define        Wrt_Rst_CR 8'h11
`define        Wrt_VME_CR 8'h12
`define        Wrt_BTO_CR 8'h13
`define       Wrt_BGTO_CR 8'h14
`define       Wrt_All_CRs 8'h15
`define       Set_Clr_CRs 8'h16
`define       Set_Inj_Err 8'h17
`define       Rst_Inj_Err 8'h18
`define     Warn_On_Shdwn 8'h19
`define  No_Warn_On_Shdwn 8'h1A
`define   Snd_Startup_Pkt 8'h1B
`define    No_Startup_Pkt 8'h1C
`define       Wrt_Ser_Num 8'h1D
`define        Rd_Ser_Num 8'h1E
`define         Wrt_CR_ID 8'h1F
//
`define      VME_Cmds 8'h20
`define      VME_Cnfg 8'h21
`define  VME_Dir_Cmds 8'h22
`define  VME_Dir_Cnfg 8'h23
//`define     UnDefined 8'h24
//                       .
//                       .
//                       .
//`define     UnDefined 8'h2F
//`define     UnDefined 8'h30
//                       .
//                       .
//                       .
//`define     UnDefined 8'hDF
`define      Wrt_Ext_FF 8'hE0
`define     Prg_Ext_Off 8'hE1
`define    Rdbk_Ext_Off 8'hE2
`define     PRst_Ext_FF 8'hE3
`define       Rd_Ext_FF 8'hE4
`define       RT_Ext_FF 8'hE5
`define     MRst_Ext_FF 8'hE6
`define    ST_MK_Ext_FF 8'hE7
`define   RST_MK_Ext_FF 8'hE8
`define Rst_Ext_Err_Cnt 8'hE9
`define Rd_Ext_Err_Cnts 8'hEA
//`define     UnDefined 8'hEB
//                       .
//                       .
//                       .
//`define     UnDefined 8'hF6
`ifdef Slave
   `define      Gen_Intr 8'hF7    //For Slave configuration only
   `define  Gen_Hard_Rst 8'hF8    //For Slave configuration only
`endif
`define  Force_Reload 8'hF9
`define Read_Drop_Mem 8'hFA
`define Read_WSOP_Mem 8'hFB
`define     Flash_R_W 8'hFC
`define  Send_N_Words 8'hFD
`define Load_User_Reg 8'hFE
`define      Loopback 8'hFF



// State definitions for Ring buff writer Controller
`define Wait_for_Trig 1'b0
`define Write_Cir_Buf 1'b1
// Terminal count to stop writing ring buffer
`define Cir_Buf_Term_Cnt  10'd143

// External FIFO Write Function codes
`define        WNoOp 3'd0
`define      MRst_FF 3'd1
`define      PRst_FF 3'd2
`define       Wrt_FF 3'd3
`define  Prg_Offsets 3'd4
`define Rst_Err_Cnts 3'd5
 
// External FIFO Read Function codes
`define        RNoOp 3'd0
`define        Rd_FF 3'd1
`define        RT_FF 3'd2
`define    SET_MK_FF 3'd3
`define    RST_MK_FF 3'd4
`define Rdbk_Offsets 3'd5
`define  Rd_Err_Cnts 3'd6
`define     Flush_FF 3'd7
 
// State definitions for External FIFO Write controller
`define         FFW_Rst 5'h00
`define    FFW_Rst_Mark 5'h01
`define        Set_FWFT 5'h02
`define         MRst_St 5'h03
`define       Hold_FWFT 5'h04
`define       MRst_Done 5'h05
`define        FFW_Idle 5'h06
`define         PRst_St 5'h07
`define       PRst_Done 5'h08
`define       Input_Rdy 5'h09
`define         Enc_BOD 5'h0A
`define        Load_BOD 5'h0B
`define       W_Valdat1 5'h0C
`define        Write_FF 5'h0D
`define         Wrt_Err 5'h0E
`define       W_Valdat2 5'h0F
`define    Load_Offsets 5'h10
`define    Wrt_Off_to_1 5'h11
`define    Wrt_Off_to_2 5'h12
`define        Prg_Done 5'h13
`define  Rst_Err_Cnt_St 5'h14

// OFFSET definitions
`define       Hi_Full_Off 5'b00001
`define       Lo_Full_Off 5'b00011
`define       Hi_Empt_Off 5'b00111
`define       Lo_Empt_Off 5'b01111
`define Hi_Un_Cor_Err_Cnt 5'b10001
`define Lo_Un_Cor_Err_Cnt 5'b10011
`define    Hi_Cor_Err_Cnt 5'b10111
`define    Lo_Cor_Err_Cnt 5'b11111

// State definitions for External FIFO Read controller
`define      FFR_Idle 5'h00
`define      RST_Mark 5'h01
`define      Set_Mark 5'h02
`define         Pause 5'h03
`define Wait_for_Read 5'h04
`define       Read_FF 5'h05
`define       ReTrans 5'h06
`define   ReTrans_Err 5'h07
`define     FF_MT_Err 5'h08
`define     FF_MK_Err 5'h09
`define     FF_Rd_Err 5'h0A
`define        Rd_Off 5'h0B
`define    Pause_Off1 5'h0C
`define    Rd_Off_Upd 5'h0D
`define    Pause_Off2 5'h0E
`define    Update_Off 5'h0F
`define     W4Capture 5'h10
`define        W4Read 5'h11
`define  Send_Offsets 5'h12
`define      Rbk_Done 5'h13
`define       W4Read2 5'h14
`define Send_Err_Cnts 5'h15
`define     Flush_BOD 5'h16
`define       W4Vldt1 5'h17
`define      Drop_Wrd 5'h18
`define       W4Vldt2 5'h19
`define    Flush_Done 5'h1A

// External FIFO Read Timeout
`define    Ex_FF_Rd_TmOut 12'hFFF

// VME Delay types
`define     No_Dly   3'd0  // No delay; VME transfer.
`define     D4nsX16  3'd1  // 4ns Res. 16 bit range
`define     D16nsX16 3'd2  // 16ns Res. 16 bit range
`define     D16usX16 3'd3  // 16us Res. 16 bit range
`define     D4nsX32  3'd4  // 4ns Res. 32 bit range
`define     D16nsX32 3'd5  // 16ns Res. 32 bit range
`define     D16usX32 3'd6  // 16us Res. 32 bit range

// VME Data Transfer Type
`define     SNGL  2'b00    // Single transfer (SNGL)
`define     BLOCK 2'b01    // Block transfer (BLOCK)
`define     RMW   2'b10    // Read-Modify-Write transfer (RMW)
`define     UNALG 2'b11    // Unaligned transfer (UNALG)

// VME Data Transfer Size
`define     Byte  2'b00
`define     Word  2'b01
`define     LWord 2'b10
`define     Eight 2'b11

// VME Address Transfer Size
`define     No_Addr  3'h0
`define     ASZ_A16  3'd1
`define     ASZ_A24  3'd2
`define     ASZ_A32  3'd3
`define     ASZ_A40  3'd4
`define     ASZ_A64  3'd5

// VME Controller error types 
`define   No_Cerr     4'h0
`define   Unkn_Addr   4'h1
`define   Unkn_Dly    4'h2
`define   Inc_Opt     4'h3
`define   RdEr_Units  4'h4
`define  RdEr_CtrlWrd 4'h5
`define   RdEr_Addr   4'h6
`define   RdEr_Dcnt   4'h7
`define   RdEr_Data   4'h8

//Address modifer encoding
//{ADDR_SZ,ACCS_TYP}
// 1-5,UD,CR,LCK,SUP,PRG
`define Usr_AM              {3'hx,5'b1xxxx}  //AM = user defined (UD_AM)
`define CR_CSR_AM           {3'hx,5'b01xxx}  //AM = 2F
//
`define A16LCK_AM       {`ASZ_A16,5'b001xx}  //AM = 2C
`define A16SUP_AM       {`ASZ_A16,5'b0001x}  //AM = 2D
`define A16non_AM       {`ASZ_A16,5'b0000x}  //AM = 29
//
`define A24LCK_AM       {`ASZ_A24,5'b001xx}  //AM = 32
`define A24SUP_AM       {`ASZ_A24,5'b0001x}  //AM = 3C-3F
`define A24non_AM       {`ASZ_A24,5'b0000x}  //AM = 38-3B
//
`define A32LCK_AM       {`ASZ_A32,5'b001xx}  //AM = 05
`define A32SUP_AM       {`ASZ_A32,5'b0001x}  //AM = 0C-0F
`define A32non_AM       {`ASZ_A32,5'b0000x}  //AM = 08-0B
//
`define A40LCK_AM       {`ASZ_A40,5'b001xx}  //AM = 35
`define A40_AM          {`ASZ_A40,5'b000xx}  //AM = 34,37
//
`define A64LCK_AM       {`ASZ_A64,5'b001xx}  //AM = 04
`define A64_AM          {`ASZ_A64,5'b000xx}  //AM = 00,01,03

// Bus Arbiter Type Definitions
`define     No_Arb  2'b00  //no bus arbitration
`define     SGL     2'b01  //Single Level Arbiter
`define     PRI     2'b10  //Priority Arbiter
`define     RRS     2'b11  //Round Robin Select Arbiter

// Bus Arbiter Priority Select FSM states
`define     Mon_BR     3'b000
`define     BR_lvl3    3'b001
`define     BR_lvl2    3'b010
`define     BR_lvl1    3'b011
`define     BR_lvl0    3'b100
`define     BGrant_Err 3'b101

// Bus Arbiter Round Robin Select FSM states
`define     RRS0 2'b00
`define     RRS1 2'b01
`define     RRS2 2'b10
`define     RRS3 2'b11

// VME input control codes
`define     VME_idle     3'd0
`define     VME_CMD_FF   3'd1
`define     VME_CNFG_FF  3'd2
`define     VME_CMD_DIR  3'd3
`define     VME_CNFG_DIR 3'd4
// VME output control codes
`define     VME_NoOp    3'd0
`define     VME_Has_Bus 3'd1

// State definitions for VME_MASTER_FSM
`define     M_idle       5'h00
`define     Capture1     5'h01
`define     Req_DTB      5'h02
`define     R1           5'h03
`define     R2A          5'h04
`define     R2B          5'h05
`define     R3A          5'h06
`define     R3B          5'h07
`define     R3C          5'h08
`define     Wait_4_DTB   5'h09
`define     Start        5'h0A
`define     Add_Phase    5'h0B
`define Wait_4_DTACK_Rel 5'h0C
`define     Sngl_Block   5'h0D
`define     RMW_1        5'h0E
`define     RMW_Read     5'h0F
`define     RMW_2        5'h10
`define     RMW_Write    5'h11
`define     M_Read       5'h12
`define     M_Write      5'h13
`define     Cycle_Done   5'h14
`define     Next_Cycle   5'h15
`define     Xfer_Done    5'h16
`define     Load_Delay   5'h17
`define     Delay        5'h18
`define     Ld_Err_Typ   5'h19
`define     Bus_Error    5'h1A

// State definitions for VME control Write state machine (VME_WRT_CTRL_FSM)
`define     VW_idle     6'h00
`define     VW_Req_Bus  6'h01
`define     Rd_FIFO     6'h02
`define     Rd_units    6'h03
`define     Ld_units    6'h04
`define     Rd_type     6'h05
`define     Ld_type     6'h06
`define     Rd_UAM      6'h07
`define     Rd_Wrd_1a   6'h08
`define     Rd_Wrd_1b   6'h09
`define     Rd_Wrd_2b   6'h0A
`define     Rd_Wrd_1c   6'h0B
`define     Rd_Wrd_2c   6'h0C
`define     Rd_Wrd_3c   6'h0D
`define     Rd_Wrd_1d   6'h0E
`define     Rd_Wrd_2d   6'h0F
`define     Rd_Wrd_3d   6'h10
`define     Rd_Wrd_4d   6'h11
`define    Ld_Add_Pause 6'h12
`define     Rd_Dcnt     6'h13
`define     Ld_Dcnt     6'h14
`define     Continue    6'h15
`define     Rd_Dat_1a   6'h16
`define     Rd_Dat_1b   6'h17
`define     Rd_Dat_2b   6'h18
`define     Rd_Dat_1d   6'h19
`define     Rd_Dat_2d   6'h1A
`define     Rd_Dat_3d   6'h1B
`define     Rd_Dat_4d   6'h1C
`define    Ld_Dat_Pause 6'h1D
`define     VPause      6'h1E
`define     Execute_Blk 6'h1F
`define     Dec_Dcnt    6'h20
`define     Execute     6'h21
`define     Dec_Units   6'h22
`define   Wait4VRd_Idle 6'h23
`define     Send_Data   6'h24
`define    Unkn_Addr_Sz 6'h25
`define    Unkn_Dly_Typ 6'h26
`define     Incomp_Opt  6'h27
`define   Rpt_Vctrl_Err 6'h28
`define   Clr_Mstr_Err1 6'h29
`define   Clr_Mstr_Err2 6'h2A
`define       Flush2BOD 6'h2B
`define    Rd_Err_Units 6'h2C
`define  Rd_Err_CtrlWrd 6'h2D
`define     Rd_Err_Addr 6'h2E
`define     Rd_Err_Dcnt 6'h2F
`define     Rd_Err_Data 6'h30

// State definitions for VME control Read  to FIFO write  (VME_RD_FFWRT_CTRL_FSM)
`define     VRd_Idle    5'h00
`define     Ld_Drbk     5'h01
`define     Ackn        5'h02
`define     Push8       5'h03
`define     Push16      5'h04
`define     Push32A     5'h05
`define     Push32B     5'h06
`define     Push64A     5'h07
`define     Push64B     5'h08
`define     Push64C     5'h09
`define     Push64D     5'h0A
`define     Push_MER    5'h0B
`define     Push_VState 5'h0C
`define     Rd_Cmplt    5'h0D
`define     Dummy8      5'h0E
`define     Dummy16     5'h0F
`define     Dummy32A    5'h10
`define     Dummy32B    5'h11
`define     Dummy64A    5'h12
`define     Dummy64B    5'h13
`define     Dummy64C    5'h14
`define     Dummy64D    5'h15
`define     Push_EOP    5'h16
`define Data_Pkt_Sent   5'h17


// State definitions for VME control Read  from FIFO   (VME_RD_FFRD_CTRL_FSM)
`define     VR_idle     3'h0
`define     Get_Bus     3'h1
`define     Pop_VFF     3'h2
`define     Wait4NMT    3'h3
`define     Last_Word   3'h4
`define     Send_Gap    3'h5
`define     GbE_Send    3'h6
`define     Rel_Bus     3'h7

// State definitions for Receive Bus controller
`define     RB_Free     4'h0
`define     TWE_RB      4'h1
`define     TWC_RB      4'h2
`define     CWC_RB      4'h3
`define     TWS_RB      4'h4
`define     Get1_BDR_RB 4'h5
`define     TWB_RB      4'h6
`define     Get2_BDR_RB 4'h7
`define     VWWB_RB     4'h8

// State definitions for FIFO Bus controller
`define     FB_Free     3'h0
`define     TWF_FB      3'h1
`define     Get_BDR_FB  3'h2
`define     TWB_FB      3'h3
`define     VWWF_FB     3'h4
`define     VWWV_FB     3'h5

// State definitions for Transmit Bus controller
`define     TB_Free     3'h0
`define     TWL_TB      3'h1
`define     TWC_TB      3'h2
`define     CWC_TB      3'h3
`define     TWT_TB      3'h4
`define     VWWL_TB     3'h5
`define     VRWV_TB     3'h6
`define     FLWFL_TB    3'h7

// State definitions for BiDirectional Reg. Controller BDR1 control.
`define     Disabled    2'h0
`define     To_FF_Bus   2'h1
`define     To_RCV_Bus  2'h2

//Bus Request codes for Traffic Controller
`define     TC_NoOp       4'h0
`define     Rd_Eth_FIFO   4'h1
`define     Lpback        4'h2
`define     Rd_RX_STAT    4'h3
`define     Rd_Ext_FIFO   4'h4
`define     Direct_VME    4'h5
`define     Spont_Rpt     4'h6
`define     TC_Cnfg_Store 4'h7
`define     Rd_Curr_Cnfg  4'h8

//Bus Request codes for VME Write Controller
`define     VMEW_NoOp     2'h0
`define     VME_fr_FIFO   2'h1
`define     VME_cnfg_save 2'h2
//`define                 2'h3

//Bus Request codes for VME Read Controller
`define     VMER_NoOp     1'b0
`define     VME_Output    1'b1

//Bus Request codes for Flash Controller
`define     FL_NoOp       1'b0
`define     FL_Output     1'b1

//Bus Request codes for Configuration Controller
`define     Cnfg_NoOp     2'h0
`define     Cnfg_Store    2'h1
`define     Cnfg_Xmit     2'h2

//FIFO Usage definitions
`define     FFNONE     2'd0
`define     VCMD       2'd1
`define     VCNFG      2'd2
`define     TSTDAT     2'd3


// Definition of TC_ERRS (errors) for Traffic Control State machine 
`define     TC_No_Err   4'h0
`define     TC_No_Data  4'h1
`define     TC_Not_Exc  4'h2
`define     TC_Unk_Fnc  4'h3
// Definition of TC_WARNS (warnings) for Traffic Control State machine 
// Actually a Startup/Shutdown signal
`define     SS_No_Warn  4'h0
`define     SS_Rld_Pndg 4'h1
// Definition of TC_INFOS (information) for Traffic Control State machine 
// Actually a Startup/Shutdown signal
`define     SS_No_Info  4'h0
`define     SS_Start_up 4'h1

// Columns of Parity part of Generator Matrix for ECC code Golay(24,12)
`define  BI1   12'h7FF
`define  BI2   12'hEE2
`define  BI3   12'hDC5
`define  BI4   12'hB8B
`define  BI5   12'hF16
`define  BI6   12'hE2D
`define  BI7   12'hC5B
`define  BI8   12'h8B7
`define  BI9   12'h96E
`define  BI10  12'hADC
`define  BI11  12'hDB8
`define  BI12  12'hB71

// Rows of Parity part of Generator Matrix for ECC code Golay(24,12)
`define  BR1   12'h7FF
`define  BR2   12'hEE2
`define  BR3   12'hDC5
`define  BR4   12'hB8B
`define  BR5   12'hF16
`define  BR6   12'hE2D
`define  BR7   12'hC5B
`define  BR8   12'h8B7
`define  BR9   12'h96E
`define  BR10  12'hADC
`define  BR11  12'hDB8
`define  BR12  12'hB71

// Rows of Identity matrix
`define  IDR1   12'h800
`define  IDR2   12'h400
`define  IDR3   12'h200
`define  IDR4   12'h100
`define  IDR5   12'h080
`define  IDR6   12'h040
`define  IDR7   12'h020
`define  IDR8   12'h010
`define  IDR9   12'h008
`define  IDR10  12'h004
`define  IDR11  12'h002
`define  IDR12  12'h001


// Flash Memory Interface State Machine Definitions  (FLC_ST)
`define  FL_Idle      5'h00
`define  W100         5'h01
`define  Read1        5'h02
`define  RelOE1       5'h03
`define  Read2        5'h04
`define  Rel_Cmp_D6   5'h05
`define  W4Data       5'h06
`define  Have_Data    5'h07
`define  Pop_n_Load   5'h08
`define  W4RdAcc      5'h09
`define  Ld_FLout_Rd  5'h0A
`define  Start_Rd     5'h0B
`define  PopPg_n_Ld   5'h0C
`define  W4WrtAcc1    5'h0D
`define  Ld_FLout_Wrt 5'h0E
`define  Start_Wrt    5'h0F
`define  W4WrtAcc2    5'h10
`define  Ld_DPW_Seq   5'h11
`define  Wrt_DPW_Seq  5'h12
`define  W4WrtAcc3    5'h13
`define  Write_Page   5'h14
`define  Poll_D7      5'h15
`define  Rel_Cmp_D7   5'h16
`define  FL_Ready     5'h17


// Dummy Flash Memory State Machine Definitions  (DP_ST)
`define  NORM_DATA    1'b0
`define  TOG_DATA_BAR 1'b1

// Flash Memory Controller Input FIFO State Machine Definitions  (FLCI_ST)
`define  FLCI_Idle   3'h0
`define  Rd_NumPg    3'h1
`define  Ld_NumPg    3'h2
`define  Rd_WPNB     3'h3
`define  Ld_WPNB     3'h4
`define  W4Rdy       3'h5
`define  Xfer2Fl     3'h6
`define  Push_TRDS   3'h7 

// Flash Memory Controller Output to GbE State Machine Definitions  (F2GC_ST)
`define  F2GC_Idle   4'h0
`define  Rd_Counts   4'h1
`define  Ld_Counts   4'h2
`define  W4AllRds    4'h3
`define  FL_Req_Bus  4'h4
`define  Xmit_Page   4'h5
`define  Xmit_Reads  4'h6
`define  Rd_Pg_Rd    4'h7 
`define  Ld_Pg_Rd    4'h8 
`define  FL_Snd_Pkt  4'h9
`define  W4Snd       4'hA
`define  Cnfg_Done   4'hB
`define  W4Cnfg      4'hC
`define  FL_Rel_Bus  4'hD

// State definitions for Flash to Configuration model FSM (F2C_FSM).
`define  F2C_Idle    2'h0
`define  F2C_Shift   2'h1
`define  F2C_Load_AC 2'h2
`define F2C_Complete 2'h3


// Masks for setting flags in control registers
`define     Test_FF_Mode_msk 32'h00000001
`define      VME_FF_Mode_msk 32'hFFFFFFFE
`define       ECC_Enable_msk 32'h00000002
`define      ECC_Disable_msk 32'hFFFFFFFD
`define      Set_Inj_Err_msk 32'h00000004
`define      Rst_Inj_Err_msk 32'hFFFFFFFB
`define    Warn_On_Shdwn_msk 32'h00000020
`define No_Warn_On_Shdwn_msk 32'hFFFFFFDF
`define  Snd_Startup_Pkt_msk 32'h00000040
`define   No_Startup_Pkt_msk 32'hFFFFFFBF

// Control register selection codes for reading out
`define   Sel_Eth_CR  3'd0
`define   Sel_Ext_CR  3'd1
`define   Sel_Rst_CR  3'd2
`define  Sel_VMEh_CR  3'd3
`define  Sel_VMEl_CR  3'd4
`define   Sel_BTO_CR  3'd5
`define  Sel_BGTO_CR  3'd6

// Configure Register ID's
`define       Eth_CR  4'd0
`define    Ext_FF_CR  4'd1
`define   Rst_Ena_CR  4'd2
`define       VME_CR  4'd3
`define       BTO_CR  4'd4
`define      BGTO_CR  4'd5
`define       ALL_CR  4'd8


// Configure Register default values

////////////////////////////////////////////////////////////////////////////////////////
//
// ETHER_CNFG(15:0)
// Ethernet configuration bits:
// Bit  Signal        Function
//  0   TXINHIBIT     Inhibits RocketIO transmission   (DISABLED)
//  1   TDIS          Disables fiber optic transmitter (DISABLED)
//  2   PASSTHRU      Received packets are transmitted back to sender without any processing
//  3   PROMISCUOUS   Accepts packets regardless of intended MAC destination.
//  4   PROTO_ENABLE  Enables a protocol header on transmitted packets.
//  5   XTEND         Enables carrier extend for packet bursting.
//  6   SPONT_PKT_ENA Enables spontaneously generated packet sending for error, warning and info reporting.
//  7   SWITCH_CNTRL  Enables front panel switch control over some of the above signals. (DISABLED)
// 8-15 RSVD          Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////
`define    Ether_Cnfg_Dflt  16'h0050

////////////////////////////////////////////////////////////////////////////////////////
//
// EXT_FIFO_CNFG(15:0)
// External FIFO configuration bits:
// Bit  Signal        Function
//  0   FF_TEST_MODE  Allows for dircet readback for testing the FIFO. (FIFO reads from the VME module are disabled)
//  1   FF_ECC_ENA    Enables error correcting codes to be used in the external FIFO
//  2   FF_INJ_ERRS   Will inject errors after ECC encoding for FIFO decode testing
// 3-15 RSVD          Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////
`define Ext_FIFO_Cnfg_Dflt  16'h0002

////////////////////////////////////////////////////////////////////////////////////////
//
// RESET_CNFG(15:0)
// Configuration bits for enabling Reset sources:
// Bit  Signal          Function
//  0   ENA_INTERNAL    Allows reprograming from an FPGA gernerated signal (on command through ethernet).
//  1   ENA_FP          Allows reprograming from front panel push button.
//  2   SYS_RESET       Allows reprograming on VME SYSRESET signal.
//  3   HARD_RESET      Allows reprograming on the custom backplane Hard Reset signal.
//  4   Ext_JTAG        Allows reprograming thru JTAG from the CF pin on the configuration PROM 
//  5   Warn_On_Shdwn   Sends a warning packet to the server when a shutdown is pending
//  6   Snd_Startup_Pkt Sends a Info packet to the server after initialization. 
// 7-15 RSVD          Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////
`define    Reset_Cnfg_Dflt  16'h0013

////////////////////////////////////////////////////////////////////////////////////////
//
// VME_CNFG(31:0)
// Configuration bits for the VME module:
// Bit  Signal          Function
//  0   VME_MSTR_ENA    Enables the VME master.
//  1   VME_SYSCLK_ENA  Enables the VME SYSCLK on the backplane (if it's a System Controller ie. slot 1)
//  2   BTO_ENA         Enables the Bus Timer module.
//  3   ARB_ENA         Enables the Arbiter module for controlling bus arbitration (if System Controller).
//  4   SLAVE_ENA       Enables the Slave module features. 
//  5   LOC_MON_ENA     Enables the Location Monitor features.
//  6   INTR_HNDLR_ENA  Enables interrupt handling.
//  7   USER_DEFS_ENA   Enables user defined buffers on J2.
// 8,9  MSTR_REQ_TYPE   Specifies the requester type for the master module.
//                          0: Release On Request (ROR)
//                          1: Release When Done (RWD)
//                          2: Fair ROR requester
//                          3: Fair RWD requester
//10,11 MSTR_REQ_LVL    Specifies the bus request level for the master module.
//12,13 ARB_TYPE        Specifies the Arbiter type.
//                          0: No Arbitration (disabled)
//                          1: SinGLe Level Arbiter (SGL)
//                          2: PRIoritized Arbiter (PRI)
//                          3: Round Robin Select (RRS)
// 14   ENA_HI_ORDER    Enable the high order bits of Address and Data Buses.
// 15   FRC_SYSRST      Force a SYSRESET.
//16-22 IH_LVLS         Specifies the Levels (1-7) the Interrupt Handler will service (7 bits).
// 23   UPD_IRQ_MSK     Update IRQ mask - restores the mask to IH_LVLS set in this register and auto resets itself to 0.
//24,25 IH_REQ_TYPE     Specifies the requester type for the Interrupt Handler.
//26,27 IH_REQ_LVL      Specifies the bus request level for the Interrupt Handler.
//28,29 IH_DT_SZ        Specifies the data transfer capability of the Interrupt Handler.
//30    SYSRST_OUT_ENA  Enable SYSRESETs out to backplane.
//31    SYSRST_IN_ENA   Enable Monitoring SYSRESETs on the backplane
//
//////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////
//  Only the following are used for the Slave version.
////////////////////////////////////////////////////////////////////////////////////////
//
// VME_CNFG(31:0)
// Configuration bits for the VME module:
// Bit  Signal          Function
//  4   SLAVE_ENA       Enables the Slave module features. 
// 9-5  CRD_ADDR        Card Address when not in VME64X slot. 
//24    INTR_ENA        Enables the Interrupter.
//25,27 IRQ_LVL         Specifies the request level assigned to this Interrupter (1-7).
//28,29 I_DT_SZ         Specifies the data transfer capability of the Interrupter.
//30-31 RSVD            Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////
`ifdef Slave
   `define      VME_Cnfg_Dflt 32'h0F000190   //Default for slave configuration
`else
   `define      VME_Cnfg_Dflt 32'hEDFF1D1F   //Default for master configuration
`endif
////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////
// VME Bus Timeout default
`ifdef Simulation
   `define       VME_BTO_Dflt 16'h00D4     // 212 (3.392 uS in 16 nS units) for simulation
`else
   `define       VME_BTO_Dflt 16'h30D4     //Default is 12,500 (16'h30D4)(200 uS in 16 nS units)
`endif
//////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////
// VME Bus Grant Timeout default
`ifdef Simulation
   `define      VME_BGTO_Dflt 16'h0139     //  313 (5.0 usec. in nS units) for simulation
`else
   `define      VME_BGTO_Dflt 16'h0C35     //Default is  3,125 (16'h0C35)(50 uS in 16 nS units)
`endif
//////////////////////////////////////////////////////////////////////////////////////

// State definitions for Configuration Module FSM
`define       Cnfg_Idle  6'h00
`define    W4ValDat_Wrt  6'h01
`define       Wrt_VME_L  6'h02
`define    W4ValDat_All  6'h03
`define     W4ValDat_SC  6'h04
`define        Mask_LD1  6'h05
`define        Mask_LD2  6'h06
`define    Set_Clr_Bits  6'h07
`define    Get_Cnfg_Num  6'h08
`define      Get_MAC_ID  6'h09
`define     Wrt_MAC_Seq  6'h0A
`define      Set_MAC_ID  6'h0B
`define     Req_Rcv_Bus  6'h0C
`define          W4Exec  6'h0D
`define     Get_MAC_Seq  6'h0E
`define      Ld_MAC_Reg  6'h0F
`define     Snd_MAC_Seq  6'h10
`define     Incr_MAC_ID  6'h11
`define    Snd_Dflt_Seq  6'h12
`define    Snd_Cnfg_Seq  6'h13
`define  Rel_Rx_Bus_MAC  6'h14
`define     MAC_PH1Done  6'h15
`define Rel_Rx_Bus_Dflt  6'h16
`define    Dflt_PH1Done  6'h17
`define Rel_Rx_Bus_Cnfg  6'h18
`define    Cnfg_PH1Done  6'h19
`define      Rcv_MAC_FL  6'h1A
`define     Rcv_Dflt_FL  6'h1B
`define     Rcv_Cnfg_FL  6'h1C
`define  Req_Tx_Bus_MAC  6'h1D
`define      Read_2_MAC  6'h1E
`define Req_Tx_Bus_Dflt  6'h1F
`define     Read_1_Dflt  6'h20
`define Req_Tx_Bus_Cnfg  6'h21
`define      Ena_Rd_Seq  6'h22
`define     Read_2_Cnfg  6'h23
`define      Load_2_MAC  6'h24
`define       Load_Dflt  6'h25
`define     Load_2_Cnfg  6'h26
`define    Wrt_MAC_Seq2  6'h27
`define        Send_MAC  6'h28
`define       Send_Dflt  6'h29
`define       Load_CRsA  6'h2A
`define      Send_CnfgA  6'h2B
`define    Incr_MAC_ID2  6'h2C
`define         Incr_Lp  6'h2D
`define     Read_1_Cnfg  6'h2E
`define     Load_1_Cnfg  6'h2F
`define       Load_CRsB  6'h30
`define      Send_CnfgB  6'h31
`define    Cnfg_Snd_Pkt  6'h32
`define      Rel_Tx_Bus  6'h33
`define        PH2_Done  6'h34
`define      Funct_Done  6'h35
`define     Get_Ser_Num_H  6'h36
`define     Get_Ser_Num_L  6'h37
`define    Snd_SerNum_Seq  6'h38
`define    SerNum_PH1Done  6'h39
`define     Rcv_SerNum_FL  6'h3A
`define Req_Tx_Bus_SerNum  6'h3B
`define     Read_1_SerNum  6'h3C
`define       Load_SerNum  6'h3D
`define       Send_SerNum  6'h3E
`define       W4ValDat_CR  6'h3F


// State definitions for Send MAC Sequence FSM
`define       SM1_Idle 4'h0
`define       SM1_Hdr1 4'h1
`define      SM1_Hdr2a 4'h2
`define     SM1_NoHdr1 4'h3
`define     SM1_NoHdr2 4'h4
`define   SM1_MC1_Load 4'h5
`define  SM1_MC1_Shift 4'h6
`define SM1_MC2_ShiftA 4'h7
`define   SM1_MC2_Load 4'h8
`define SM1_MC2_ShiftB 4'h9
`define      SM1_Shift 4'ha
`define       SM1_Done 4'hb

// State definitions for Send Default Sequence FSM
`define        SD1_Idle 3'h0
`define        SD1_Hdr1 3'h1
`define       SD1_Hdr2b 3'h2
`define   SD1_Dflt_Load 3'h3
`define  SD1_Dflt_Shift 3'h4
`define        SD1_Done 3'h5

// State definitions for Send Serial Number Sequence FSM
`define        SSN1_Idle 3'h0
`define        SSN1_Hdr1 3'h1
`define       SSN1_Hdr2d 3'h2
`define SSN1_SerNum_Load 3'h3
`define       SSN1_Shift 3'h4
`define        SSN1_Done 3'h5

// State definitions for Send Cnfg Sequence FSM
`define         SC1_Idle 4'h0
`define         SC1_Hdr1 4'h1
`define        SC1_Hdr2c 4'h2
`define   SC1_Cnfg_LoadA 4'h3
`define  SC1_Cnfg_ShiftA 4'h4
`define  SC1_Cnfg_ShiftB 4'h5
`define  SC1_Cnfg_ShiftC 4'h6
`define   SC1_Cnfg_LoadB 4'h7
`define         SC1_Done 4'h8


//// For simulations of Config module only
//`define    Exc_Idle 3'o0
//`define   Exc_Wait1 3'o1
//`define   Exc_Hold1 3'o2
//`define   Exc_Wait2 3'o3
//`define   Exc_Hold2 3'o4
//// For simulations of Config module only
//`define        PU_Idle 3'o0
//`define   PU_Rstr_Dflt 3'o1
//`define       PU_Wait1 3'o2
//`define   PU_Rstr_MACs 3'o3
//`define       PU_Wait2 3'o4
//`define   PU_Rstr_Cnfg 3'o5
//`define PU_Clr_Startup 3'o6

//Definition of MAC types
`define   Device_MAC 3'o0
`define   Mcast1_MAC 3'o1
`define   Mcast2_MAC 3'o2
`define   Mcast3_MAC 3'o3
`define Dflt_Srv_MAC 3'o4


// MAC address firmware defaults (Type,Addr,MAC_Wrd):
`define   Dflt_MAC0 21'h00AABB
`define   Dflt_MAC1 21'h01CCDD
`define   Dflt_MAC2 21'h02EEFF
`define   Dflt_MAC3 21'h04FFFF
`define   Dflt_MAC4 21'h05FFFF
`define   Dflt_MAC5 21'h06FFFE
`define   Dflt_MAC6 21'h08FFFF
`define   Dflt_MAC7 21'h09FFFF
`define   Dflt_MAC8 21'h0AFFFD
`define   Dflt_MAC9 21'h0CFFFF
`define   Dflt_MACA 21'h0DFFFF
`define   Dflt_MACB 21'h0EFFFC
`define   Dflt_MACC 21'h10FFFF
`define   Dflt_MACD 21'h11FFFF
`define   Dflt_MACE 21'h12FFFF

// Packet type definitions
`define   Norm_Pkt 3'd0  //Normal data packet
`define   INTR_Pkt 3'd1  //Interrupt Status/ID packet
`define   INFO_Pkt 3'd2  //Informational packet
`define   WARN_Pkt 3'd3  //Warning packet
`define   ERR_Pkt  3'd4  //Error packet

// Report Sources   for INFOs, WARNs, and ERRs
`define Misc          4'h0  //No specified source
`define VME_Ctrl      4'h1  //VME Controller module
`define VME_Master    4'h2  //VME Master module
`define VME_Rdbk      4'h3  //VME Readback module
`define VME_IH        4'h4  //VME Interrupt module
`define VME_Slv       4'h5  //VME Slave module
`define VME_Arb       4'h6  //VME Arbiter module
`define Ext_FIFO_mod  4'h7  //External FIFO module
`define Eth_Rcv       4'h8  //Ethernet Receive module
`define Eth_Trns      4'h9  //Ethernet Transmit module
`define JTAG_mod      4'hA  //JTAG Interface
`define Flash_mod     4'hB  //Flash module
`define Config_mod    4'hC  //Configuration module
`define BTC_mod       4'hD  //Bus Traffic Control module
`define Rst_Hndlr     4'hE  //Reset Handler module
`define Strtup_Shtdwn 4'hF  //Start-up / Shutdown module

// Sources of errors and data in VME master
`define No_Req        4'h0  //No Requests
`define V_Mstr_Err    4'h1  //VME Master Error
`define V_IH_Err      4'h2  //VME Interrupt Handler Error
`define V_Ctrl_Err    4'h3  //VME Control Error
`define V_Mstr_Dav    4'h4  //VME Master Data Available
`define V_IH_Dav      4'h5  //VME Interrupt Handler Data Available
`define V_IH_Warn     4'h6  //VME Interrupt Handler Warning

// State definitions for Interrupt Handler Module FSM
`define IH_Idle       4'h0
`define IH_Cap_IRQ    4'h1
`define IH_Req_DTB    4'h2
`define IH_Wait4DTB   4'h3
`define IH_Start      4'h4
`define IACK_Cycle    4'h5
`define Status_ID     4'h6
`define IACK_Done     4'h7
`define IH_Ld_Err     4'h8
`define IH_Bus_Err    4'h9
`define IH_Rel_BR     4'hA
`define Mask_Intr     4'hB
`define IH_Ld_Warn    4'hC
`define IH_Warn       4'hD

// Time out for releasing Interrupt Request lines.
`define IACK_Time_Out  7'h7F      // 500 ns for ROAK interrupters (ROAR interrupters not supported)




////////////////////////////////////////////////////////////////////////////////////////
//  The following (from here to end) are used only for the Slave version.
////////////////////////////////////////////////////////////////////////////////////////
`ifdef Slave

   // State definitions for Interrupter Module FSM
   `define Intr_Idle    3'h0
   `define Set_IRQ      3'h1
   `define Drv_IACKOUT  3'h2
   `define Rel_IACKOUT  3'h3
   `define Chk_SZ       3'h4
   `define Send_Status  3'h5
   `define Rel_IRQ      3'h6
   `define I_Bus_Err    3'h7

   // Masks for address modes
   `define A16_Mask 64'h00000000000007FF
   `define A24_Mask 64'h000000000007FFFF
   `define A32_Mask 64'h0000000007FFFFFF
   `define A40_Mask 64'h00000007FFFFFFFF
   `define A64_Mask 64'h07FFFFFFFFFFFFFF
   // Slave board device addresses
   `define MEMA16_a  64'h00000000000000XX
   `define MEMA16_b  64'h00000000000001XX
   `define MEMA16_c  64'h00000000000002XX
   `define MEMA16_d  64'h00000000000003XX
   `define MEMA16_e  64'h00000000000004XX
   `define MEMA16_f  64'h00000000000005XX
   `define MEMA16_g  64'h00000000000006XX
   `define MEMA16_h  64'h00000000000007XX
   `define MEMA32_a  64'h00000000000008XX
   `define MEMA32_b  64'h00000000000009XX
   `define MEMA32_c  64'h0000000000000AXX
   `define MEMA32_d  64'h0000000000000BXX
   `define MEMA32_e  64'h0000000000000CXX
   `define MEMA32_f  64'h0000000000000DXX
   `define MEMA32_g  64'h0000000000000EXX
   `define MEMA32_h  64'h0000000000000FXX
   `define MEMB8     64'h0000000000001XXX
   `define MEMB64    64'h0000000000002XXX
   `define FIFO_a    64'h0000000000003000
   `define FIFO_b    64'h0000000000003002
   `define FIFO_c    64'h0000000000003004
   `define FIFO_d    64'h0000000000003006
   `define Rnd_Rbn   64'h0000000000003008
   // Slave board device definition codes
   `define No_Such_Dev 4'h0
   `define MEMA16_Dev  4'h1
   `define MEMA32_Dev  4'h2
   `define MEMB8_Dev   4'h3
   `define MEMB64_Dev  4'h4
   `define FIFO_Dev    4'h5
   `define Rnd_Rbn_Dev 4'h6

   // Data Transfer Types (DTT)
   //
   //   0    unsupported combination           UnSup
   //   1    D08      Byte(0)                  D08_B0
   //   2    D08      Byte(1)                  D08_B1
   //   3    D08      Byte(2)                  D08_B2
   //   4    D08      Byte(3)                  D08_B3
   //   5    D16      Byte(0-1)                D16_B01
   //   6    D16      Byte(2-3)                D16_B23
   //   7    D32      Byte(0-3)                D32_B03
   //   8    MD32     Byte(0-3)                MD32_B03
   //   9    MBLT(D64)Byte(0-7)                D64_B07
   //   A    D32:UAT  Byte(0-2)                UAT_B02
   //   B    D32:UAT  Byte(1-3)                UAT_B13
   //   C    D32:UAT  Byte(1-2)                UAT_B12
   `define UnSup     4'h0
   `define D08_B0    4'h1
   `define D08_B1    4'h2
   `define D08_B2    4'h3
   `define D08_B3    4'h4
   `define D16_B01   4'h5
   `define D16_B23   4'h6
   `define D32_B03   4'h7
   `define MD32_B03  4'h8
   `define D64_B07   4'h9
   `define UAT_B02   4'hA
   `define UAT_B13   4'hB
   `define UAT_B12   4'hC


   // State definitions for Slave Response FSM
   `define  Slv_Idle     5'h00
   `define  Cap_AM       5'h01
   `define  Dcd_Addr     5'h02
   `define  Select_Dev   5'h03
   `define  AD_Hndshk    5'h04
   `define  W_4_DS_1     5'h05
   `define  W_4_DS_2     5'h06
   `define  Rpt_BERR     5'h07
   `define  DTACK_1      5'h08
   `define  DTACK_2      5'h09
   `define  Get_DTT      5'h0A
   `define  Ld_A0        5'h0B
   `define  Cap_Dat      5'h0C
   `define  W_4_Status   5'h0D
   `define  W_4_ValDat   5'h0E
   `define  DTACK_3      5'h0F
   `define  W_4_DS_3     5'h10
   `define  Inc_Addr     5'h11
   `define  Clr_All      5'h12


   // State definitions for Memory FSM
   `define  Mem_Idle     3'h0
   `define  Chk_Sz_Wrt   3'h1
   `define  Wrt_Ackn     3'h2
   `define  Parity_Chk   3'h3
   `define  Data_Valid   3'h4
   `define  Dat_Err      3'h5

`endif
